@@ -1,4 +1,5 @@
 import importlib.metadata as importlib_metadata
+import time
 from typing import Generator, List, Optional, Tuple, Union
 
 import grpc
@@ -21,8 +22,10 @@ from nibiru_proto.proto.cosmos.tx.v1beta1 import service_pb2_grpc as tx_service_
 
 import nibiru.query_clients
 from nibiru.network import Network
+from nibiru.utils import init_logger
 
 DEFAULT_TIMEOUTHEIGHT = 20  # blocks
+GITHUB_COMMIT_HASH_LEN = 40
 
 
 class GrpcClient:
@@ -31,7 +34,17 @@ class GrpcClient:
         network: Network,
         insecure=False,
         credentials: grpc.ChannelCredentials = None,
+        bypass_version_check: bool = False,
     ):
+        """
+        _summary_
+
+        Args:
+            network (Network): The network object
+            insecure (bool, optional): Wether the network should use ssl or not. Defaults to False.
+            credentials (grpc.ChannelCredentials, optional): Ssl creds. Defaults to None.
+            bypass_version_check (bool, optional): Wether to bypass the check for correct version of the chain/py-sdk
+        """
 
         # load root CA cert
         if not insecure:
@@ -60,11 +73,46 @@ class GrpcClient:
         self.pricefeed = nibiru.query_clients.PricefeedQueryClient(self.chain_channel)
         self.perp = nibiru.query_clients.PerpQueryClient(self.chain_channel)
         self.vpool = nibiru.query_clients.VpoolQueryClient(self.chain_channel)
+        self.epoch = nibiru.query_clients.EpochQueryClient(self.chain_channel)
+        self.auth = nibiru.query_clients.AuthQueryClient(self.chain_channel)
 
-        # Assert that we use the correct version of the chain
-        nibiru_proto_version = importlib_metadata.version("nibiru_proto")
+        if not bypass_version_check:
+            self.assert_compatible_versions(
+                nibiru_proto_version=importlib_metadata.version("nibiru_proto"),
+                chain_nibiru_version=str(self.get_version()),
+            )
 
-        assert nibiru_proto_version.split(".") >= self.get_version()[1:].split(".")
+    def assert_compatible_versions(self, nibiru_proto_version, chain_nibiru_version):
+        """
+        Assert that this version of the python sdk is compatible with the chain.
+        If you run the chain from a non tagged release, the version query will be returning something like
+        master-6a315bab3db46f5fa1158199acc166ed2d192c2f. Otherwise, it should be for example `v0.14.0`.
+
+        If the chain is running a custom non tagged release, you are free to use the python sdk at your own risk.
+        """
+        if nibiru_proto_version[0] == "v":
+            nibiru_proto_version = nibiru_proto_version[1:]
+        if chain_nibiru_version[0] == "v":
+            chain_nibiru_version = chain_nibiru_version[1:]
+
+        if len(chain_nibiru_version) >= GITHUB_COMMIT_HASH_LEN:
+            logger = init_logger("client-logger")
+            logger.warning(
+                f"The chain is running a custom release from branch/commit {chain_nibiru_version}. "
+                "We bypass the compatibility assertion"
+            )
+            logger.warning(
+                f"The chain is running a custom release from branch/commit {chain_nibiru_version}"
+            )
+        else:
+            nibiru_proto_version = tuple(map(int, nibiru_proto_version.split(".")))
+            chain_nibiru_version = tuple(map(int, chain_nibiru_version.split(".")))
+
+            error_string = (
+                f"Version error, Python sdk runs with nibiru protobuf version {nibiru_proto_version}, but the "
+                f"remote chain is running with version {chain_nibiru_version}"
+            )
+            assert nibiru_proto_version >= chain_nibiru_version, error_string
 
     def close_chain_channel(self):
         self.chain_channel.close()
@@ -73,6 +121,14 @@ class GrpcClient:
         block = self.get_latest_block()
         self.timeout_height = block.block.header.height + DEFAULT_TIMEOUTHEIGHT
 
+    def wait_for_next_block(self):
+        """
+        Wait for a block to be written
+        """
+        current_block = self.get_latest_block().block.header.height
+        while self.get_latest_block().block.header.height < current_block + 1:
+            time.sleep(0.5)
+
     def get_block_by_height(
         self, height: int
     ) -> tendermint_query.GetBlockByHeightResponse:
